import { SBaseEditScene, SBasePipe, } from "./../big-edit";
import { SGraphEdit, } from "./../edit";
import { SMouseEvent } from "@persagy-web/base/lib";
import { SGraphSelectContainer, SLineStyle } from "@persagy-web/graph";
import { SItemStatus } from "@persagy-web/big/lib/enums/SItemStatus";
import { rgbaNum } from "./../big-edit/until";
// 引入命令
import { SGraphAddCommand } from "./../edit/commands/SGraphAddCommand"
import { SColor, SFont, SArrowStyleType } from '@persagy-web/draw/lib';
export class PTopoScene extends SBaseEditScene {
    /** 图例数据 */
    legendObj: any = null;
    constructor() {
        super()
        // // 选择绑定选额item事件
        this.selectContainer.connect("listChange", this, this.listChange);
    }


    /**
     * 选中返回的选中item回调方法
     *
     * @param   event   鼠标事件参数
     */
    listChange(list: any): void {
        const itemList: any = []
        console.log('list', list)
        list.itemList.forEach((item: any) => {
            if ((item instanceof SGraphEdit) && !(item instanceof SGraphSelectContainer)) {
                itemList.push(item)
            }
        })
        this.emitChoice(itemList);
    } // Function listChange()

    /**
     * 选中返回的选中item回调方法(用于场景的外部调用)
     *
     * @param   list   选中的item数组
    */
    emitChoice(list: any) {
    } // Function emitChoice()

    /**
     * 鼠标左键按下
     *
     * @param   event   鼠标事件参数
     */
    onMouseDown(event: SMouseEvent): any {
        this.vueOnMouseDown(event) //外部调用
        if (this.grabItem) {
            if (this.grabItem instanceof SBasePipe) {
                this.setTipeEndanchor(event)
                return true;
            }
            return this.grabItem.onMouseDown(event);
        }
        if (this.editCmd == "EditBaseLine") {
            this.addPolyLineArrow(event);
            this.clearCmdStatus();
        } else if (this.editCmd == "EditBasePolyLine") {
            this.addPolyLine(event);
            this.clearCmdStatus();
        } else if (this.editCmd == "EditBasetext") {
            this.addTextItem(event);
            this.clearCmdStatus();
        }
        else if (this.editCmd == "BaseExplain") {
            this.addExplainItem(event);
            this.clearCmdStatus();
        } else if (this.editCmd == "EditBaseImage") {
            this.addImageItem(event)
            this.clearCmdStatus();
        }
        else if (this.editCmd == "EditBasePolygon") {
            this.addPolygonItem(event);
            this.clearCmdStatus();
        } else if (this.editCmd == "EditBaseRect") {
            this.addRectItem(event)
            this.clearCmdStatus();
        } else if (this.editCmd == "EditBaseTriangle") {
            this.addTriangleItem(event)
            this.clearCmdStatus();
        } else if (this.editCmd == "EditBaseCircle") {
            this.addCircleItem(event)
            this.clearCmdStatus();
        }
        else if (this.editCmd == "EditBaseArrows") {
            this.addPolygonArrow(event)
            this.clearCmdStatus();
        }
        else if (this.editCmd == "wantou" || this.editCmd == "santong" || this.editCmd == "sitong") {
            this.addPipeUninTool(event, this.editCmd)
            this.clearCmdStatus();
        }
        else if (this.editCmd == "EditBasePipe") {
            // this.addBasePipe(event);
            this.addBaseIconTextItem(event)
            this.clearCmdStatus();
        } else if (this.editCmd == "EditEuqipment" || this.legendObj) {
            this.addEuqipment(event, this.legendObj)
            this.clearCmdStatus();
        }
        else if (this.editCmd == "") {
            super.onMouseDown(event);
        }
    } // Function onMouseDown()

    /**
     * 鼠标右键事件
     *
     * @param   event   鼠标事件参数
     */
    onContextMenu(event: SMouseEvent): boolean {
        if (!super.onContextMenu(event)) {
            this.getItem(null, [event])
        }
        return true
    } // Function onContextMenu()

    /**
     * 修改 item 样式，数据等方法
     * @param styletype string 修改样式类型
     * @param changeStyle 更改样式数据
     * @param itemList? SGraphEdit[] 如果不传入默认使用选择器中选中得item
     */
    updateStyle(styletype: string, changestyle: any, itemList?: SGraphEdit[]): void {

        // 如果未传入需要修改样式的item,默认取选择器中的item
        let List = null;
        if (itemList && itemList.length) {
            List = itemList;
        } else {
            List = this.selectContainer.itemList;
        };
        let styleValue: any
        if (styletype == "strokeColor" || styletype == "backgroundColor" || styletype == "fillColor") {
            const colorlist = rgbaNum(changestyle)
            styleValue = new SColor(Number(colorlist[0]), Number(colorlist[1]), Number(colorlist[2]), colorlist[3] * 255);
        } else if (styletype == "lineStyle") {
            styleValue = SLineStyle[changestyle]
        } else if (styletype == "begin" || styletype == "end") {
            styleValue = SArrowStyleType[changestyle]
        } else if (styletype == "font") {
            styleValue = new SFont("sans-serif", changestyle)
        } else {
            styleValue = changestyle
        }
        List.forEach((item: SGraphEdit, index: number) => {
            if (item instanceof SGraphSelectContainer) {
                return
            }
            item[styletype] = styleValue;
        })

    } // Function updateStyle()

    /**
     * item 创建完成后回调
     *
     * @param   event   鼠标事件参数
     */
    finishCreated(item: SGraphEdit): void {
        this.grabItem = null;
        item.status = SItemStatus.Normal;
        this.undoStack.push(new SGraphAddCommand(this, item));
        this.selectContainer.clear();
        this.selectContainer.toggleItem(item);
    } // Function finishCreated()

    /**
     * 修改 cmdstatus 函数；常在在业务中调用
     *
     */
    clearCmdStatus() {
    } // Function clearCmdStatus()

    /**
     * 获取item (常用与场景外的调用F)
     * @param  event SMouseEvent 鼠标事件
     * @param  item SGraphEdit|null 返回item
     *
     */
    getItem(item: SGraphEdit | null, event: SMouseEvent[]) {
        // alert('onContextMenu')
    } // Function getItem()

    /**
     * 获取item (常用与场景外的调用F)
     * @param  event SMouseEvent 鼠标事件
     * @param  item SGraphEdit|null 返回item
     *
     */
    vueOnMouseDown(event: SMouseEvent) {
    } // Function vueOnMouseDown()

    /**
     * 设置 item 状态
     */
    setItemStatus() {
        const List = this.selectContainer.itemList[0];
        if (List && List instanceof SGraphEdit) {
            if (List.status == SItemStatus.Normal) {
                List.status = SItemStatus.Edit;
                this.grabItem = List;
            } else {
                List.status = SItemStatus.Normal;
                this.grabItem = null
            }
        }
    } // Function setItemStatus()

    /**
     * 获取item (常用与场景外的调用F)
     *
     * @params isAll 是否为全部item数据
     * @return obj 返回保存的数据接口
     */
    save(isAll: boolean = true) {
        if (!this.view) return;
        const Marktype: string[] = ['BasePolygon', 'BaseLine', 'BaseText', 'BaseExplain', 'BaseImage', 'BaseCircle', 'BaseArrow', 'BaseTriangle', 'BaseRect', 'BaseArrowPolygon'];
        const NodeType: string[] = ['BaseEquipment'];
        const RelationType: string[] = [];
        const markers: any = [];   /**图例节点 */  // 与工程信息无关的标识对象（增加文本注释，图上的图片说明）
        const nodes: any = [];   /**图例节点 */  // 与工程信息无关的标识对象（增加文本注释，图上的图片说明）
        const relations: any = [];   /**图例节点 */  // 与工程信息无关的标识对象（增加文本注释，图上的图片说明）
        let nodeList;
        if (isAll) {
            nodeList = this.root.children;
        } else {
            nodeList = this.selectContainer.itemList;
        };
        nodeList.forEach(item => {
            if ((item instanceof SGraphEdit) && !(item instanceof SGraphSelectContainer)) {
                // 添加节点数据
                if (item.data && Marktype.includes(item.data.properties.type)) {
                    markers.push(item.toData())
                }
                if (item.legendData && NodeType.includes(item.legendData.properties.type)) {
                    nodes.push(item.toData())
                }
                if (item.relationData && RelationType.includes(item.data.properties.type)) {
                    relations.push(item.toData())
                }
            }
        });
        return {
            markers,
            nodes,
            relations
        }
    } // Function save()
}